
#include "Maze.hpp"

Maze::Maze(void) {
  oled.begin();
  lcd.begin();
  ss.init();
  
  pinMode(BUZZER, OUTPUT);

  //
  lcd.print("KMT AMU 2019");
  lcd.setCursor(0, 1);
  lcd.print("Labirynt");
  
  Init();
  ShowBoard();
}

void Maze::Init(void) {
  pinMode(KEY_UP, INPUT); 
  pinMode(KEY_DOWN, INPUT);
  pinMode(KEY_LEFT, INPUT);
  pinMode(KEY_RIGHT, INPUT);
  const char* m = 
  "###############################"
  "a     #   #     #           # #"
  "##### # # # ### # ### ##### # #"
  "#   #   #   #     #   #   #   #"
  "# # ######### ######### # #####"
  "# #       #   #   #   # # #   #"
  "# ######### ##### # # ### # # #"
  "# #   #     #     # #       # #"
  "### # # ##### ##### ######### #"
  "#   #   # #   #     #         #"
  "# ####### # ### ##### #########"
  "# # #     #   #   #   #       #"
  "# ### ########### # ##### ### #"
  "#                 #       #   b"
  "###############################";

  memset(board,  0, sizeof(board));

  for (int i = 0; i < 31; i++)
    for (int j = 0; j < 15; j++) {
      char ch = m[j * 31 + i];
      board[j][i] =  ch == '#';

      if (ch == 'a') {
        startx = i;
        starty = j;
      }

      if (ch == 'b') {
        stopx = i;
        stopy = j;
      }
    }

  //
  playerx = startx;
  playery = starty;

}


void Maze::DrawSquare(int col, int row, bool state) {
  for (int i = 0; i < SQUARE_SIZE; i++)
    for (int j = 0; j < SQUARE_SIZE; j++)
      oled.setPixel(col * SQUARE_SIZE + i, row * SQUARE_SIZE + j, state);
}


void Maze::ShowBoard(void) {
  oled.clear(false);
  
  for (int row = 0; row < BOARD_HEIGHT; row++)
    for (int col = 0; col < BOARD_WIDTH; col++)
      DrawSquare(col, row, board[row][col]);

  oled.writeRect(playerx * SQUARE_SIZE + 1, playery * SQUARE_SIZE + 1, 2, 2, false);

  oled.setPixel(startx * SQUARE_SIZE + 1, starty * SQUARE_SIZE + 1, true);
  oled.setPixel(startx * SQUARE_SIZE + 2, starty * SQUARE_SIZE + 2, true);

  oled.setPixel(stopx * SQUARE_SIZE + 2, stopy * SQUARE_SIZE + 1, true);
  oled.setPixel(stopx * SQUARE_SIZE + 1, stopy * SQUARE_SIZE + 2, true);


  ss.displayDigit(playery % 10, 0);
  ss.displayDigit(playery / 10, 1);
  ss.displayDigit(playerx % 10, 2);
  ss.displayDigit(playerx / 10, 3);

  oled.renderAll();
}

bool Maze::Step(int dx, int dy)
{
  int nx = playerx + dx;
  int ny = playery + dy;
  if (nx < 0 || nx >= BOARD_WIDTH)
    return false;
  if (ny < 0 || ny >= BOARD_HEIGHT)
    return 0;

  if (board[ny][nx]) {
//    delay(200);

    digitalWrite(BUZZER, 1);  delay(10);  digitalWrite(BUZZER, 0); delay(50);
    
    return false;
  }

  playerx = nx;
  playery = ny;

  ShowBoard();
  delay(200);

  if (playerx == stopx && playery == stopy)
  {
    
    digitalWrite(BUZZER, 1);  delay(20);  digitalWrite(BUZZER, 0); delay(100);
    digitalWrite(BUZZER, 1);  delay(100);  digitalWrite(BUZZER, 0); delay(200);
    digitalWrite(BUZZER, 1);  delay(100);  digitalWrite(BUZZER, 0); delay(200);
    digitalWrite(BUZZER, 1);  delay(20);  digitalWrite(BUZZER, 0); delay(100);
    
    return false;
  }
  
  return true;
}

void Maze::GoUp(void) {
  while (Step(0, -1));
}

void Maze::GoDown(void) {
  while (Step(0, 1));
}

void Maze::GoLeft(void) {
  while (Step(-1, 0));
}

void Maze::GoRight(void) {
  while (Step(1, 0));
  
}
